@@ -22,7 +22,8 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <time.h>
-#include <unique/unique.h>
+
+#define QUITCOUNT_RESPONSE_PREFERENCES 234
 
 time_t stop_date = (time_t)0;
 int num_per_day;
@@ -254,12 +255,10 @@ static gboolean check_anniversary(gpointer data)
 		str = g_strdup_printf(_("You quit since %d days!"), num_day);
 		anniversary = TRUE;
 	}
-#if GTK_CHECK_VERSION(2,16,0)
 	if (str)
 		gtk_status_icon_set_tooltip_text(icon, str);
 	else
 		gtk_status_icon_set_has_tooltip(icon, FALSE);
-#endif
 	g_free(str);
 
 	set_icon(anniversary);
@@ -317,7 +316,7 @@ void show_config(gpointer object, gpointer user_data)
 	GtkWidget *hbox;
 	time_t now;
 	struct tm *ctime;
-	int d, m, y;
+	guint d, m, y;
 
 	if (stop_date != 0)
 		now = stop_date;
@@ -338,11 +337,7 @@ void show_config(gpointer object, gpointer user_data)
 			GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT,
 			NULL);
 			
-#if GTK_CHECK_VERSION(2,14,0)
 	box = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
-#else
-	box = GTK_DIALOG(dialog)->vbox;
-#endif
 	
 	table = gtk_table_new(3, 2, FALSE);
 	
@@ -448,8 +443,10 @@ void show_config(gpointer object, gpointer user_data)
 	gtk_dialog_run(GTK_DIALOG(dialog));
 	
 	gtk_calendar_get_date(GTK_CALENDAR(stop_date_calendar),
-				&ctime->tm_year, &ctime->tm_mon, &ctime->tm_mday);
-	ctime->tm_year -= 1900;
+				&y, &m, &d);
+	ctime->tm_year = y - 1900;
+	ctime->tm_mon  = m;
+	ctime->tm_mday = d;
 	
 	ctime->tm_hour = ctime->tm_min = ctime->tm_sec = 1;
 	
@@ -474,10 +471,6 @@ static void do_quit(gpointer object, gpointer user_data)
 	gtk_main_quit();
 }
 
-static void open_link(gpointer object, const gchar *link, gpointer data)
-{
-}
-
 static void do_about(gpointer object, gpointer user_data)
 {
 	GtkWidget *dialog = gtk_about_dialog_new();
@@ -487,8 +480,6 @@ static void do_about(gpointer object, gpointer user_data)
 				  NULL};
 	const gchar *authors[] = {"Colin Leroy <colin@colino.net>", NULL};
 	
-	gtk_about_dialog_set_url_hook((GtkAboutDialogActivateLinkFunc)open_link, NULL, NULL);
-
 	gtk_about_dialog_set_artists(GTK_ABOUT_DIALOG(dialog), (const gchar **)artists);
 	gtk_about_dialog_set_authors(GTK_ABOUT_DIALOG(dialog), (const gchar **)authors);
 	gtk_about_dialog_set_copyright(GTK_ABOUT_DIALOG(dialog), 
@@ -510,16 +501,17 @@ static void do_activate(gpointer object, gpointer user_data)
 	GtkWidget *dialog = gtk_dialog_new_with_buttons(
 			_("QuitCount statistics"),
 			NULL, GTK_DIALOG_MODAL,
+			GTK_STOCK_PREFERENCES, QUITCOUNT_RESPONSE_PREFERENCES,
 			GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT,
 			NULL);
-#if GTK_CHECK_VERSION(2,14,0)
+	gint response;
+
 	GtkWidget *box = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
-#else
-	GtkWidget *box = GTK_DIALOG(dialog)->vbox;
-#endif
 	GtkWidget *table, *label;
 	gint row = 0;
 
+	gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT);
+
 	if (num_day > 7)
 		table = gtk_table_new(6, 2, FALSE);
 	else
@@ -679,20 +671,30 @@ static void do_activate(gpointer object, gpointer user_data)
 	gtk_box_pack_start(GTK_BOX(box), wrap(table, 
 		_("<span size='x-large'><b>QuitCount - statistics</b></span>")), FALSE, FALSE, 10);
 	gtk_widget_set_size_request(dialog, 500, -1);
-	gtk_dialog_run(GTK_DIALOG(dialog));
+	response = gtk_dialog_run(GTK_DIALOG(dialog));
 	gtk_widget_destroy(dialog);
+
+	if (response == QUITCOUNT_RESPONSE_PREFERENCES)
+		show_config(NULL, NULL);
 }
 
 static void do_popup_menu(GtkStatusIcon *i, guint button, guint activate_time, 
 			  gpointer user_data)
 {
 	GtkWidget *menu = gtk_menu_new();
-	GtkWidget *item = gtk_image_menu_item_new_from_stock(GTK_STOCK_PREFERENCES, NULL);
+	GtkWidget *item;
 	GtkWidget *img;
 	gchar *path = g_strconcat(ICONSDIR, G_DIR_SEPARATOR_S, PACKAGE, ".svg", NULL);
 	gint w,h;
 	GdkPixbuf *pixbuf;
 	
+	item = gtk_image_menu_item_new_from_stock(GTK_STOCK_PREFERENCES, NULL);
+
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+	gtk_widget_show(item);
+	g_signal_connect(G_OBJECT(item), "activate", 
+			 G_CALLBACK(show_config), NULL);
+
 	if (!gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &w, &h)) {
 		w = 16;
 		h = 16;
@@ -702,11 +704,6 @@ static void do_popup_menu(GtkStatusIcon *i, guint button, guint activate_time,
 	g_free(path);	
 	img = gtk_image_new_from_pixbuf(pixbuf);
 	g_object_unref(G_OBJECT(pixbuf));
-	
-	gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-	gtk_widget_show(item);
-	g_signal_connect(G_OBJECT(item), "activate", 
-			 G_CALLBACK(show_config), NULL);
 
 	item = gtk_image_menu_item_new_with_label(_("Statistics"));
 	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), img);
@@ -733,51 +730,48 @@ static void do_popup_menu(GtkStatusIcon *i, guint button, guint activate_time,
 			button, activate_time);
 }
 
-UniqueResponse unique_msg_cb(UniqueApp *app, gint command, 
-			     UniqueMessageData *msg, guint time_,
-			     gpointer user_data)
+void run_quitcount(GtkApplication *app)
 {
-	if (command == 1 && stop_date != (time_t)0) {
+	GList *windows = gtk_application_get_windows(app);
+	if (windows == NULL) {
+		GtkWidget *dummy = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+
+		gtk_window_set_application(GTK_WINDOW(dummy), app);
+
+		set_icon(FALSE);
+
+		if (!load_config()) {
+			currency = g_strdup("€");
+			show_config(NULL, NULL);
+		}
+		install_anniversary_watch();
+	
+		g_signal_connect(G_OBJECT(icon), "popup-menu", 
+				 G_CALLBACK(do_popup_menu), NULL);
+		g_signal_connect(G_OBJECT(icon), "activate", 
+				 G_CALLBACK(do_activate), NULL);
+	} else if (stop_date != (time_t)0) {
 		do_activate(NULL, NULL);
 	}
-	return UNIQUE_RESPONSE_OK;
 }
 
 int main(int argc, char *argv[])
 {
-	UniqueApp *app = NULL;
-	
+	GtkApplication *app = NULL;
+	int status;
+
 	gtk_init(&argc, &argv);
 	
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	bind_textdomain_codeset (PACKAGE, "UTF-8");
 	textdomain(PACKAGE);
 
-	app = unique_app_new("net.sf.quitcount", NULL);
-	
-	unique_app_add_command(app, "popup", 1);
-	
-	g_signal_connect(G_OBJECT(app), "message-received",
-			 G_CALLBACK(unique_msg_cb), NULL);
+	app = gtk_application_new("net.sf.quitcount", 0);
+	g_signal_connect(G_OBJECT(app), "activate",
+			 G_CALLBACK(run_quitcount), NULL);
 
-	if (unique_app_is_running(app)) {
-		unique_app_send_message(app, 1, NULL);
-		return 0;
-	}
-	
-	set_icon(FALSE);
+	status = g_application_run(G_APPLICATION(app), argc, argv);
+	g_object_unref(app);
 
-	if (!load_config()) {
-		currency = g_strdup("€");
-		show_config(NULL, NULL);
-	}
-	install_anniversary_watch();
-	
-	g_signal_connect(G_OBJECT(icon), "popup-menu", 
-			 G_CALLBACK(do_popup_menu), NULL);
-	g_signal_connect(G_OBJECT(icon), "activate", 
-			 G_CALLBACK(do_activate), NULL);
-
-	gtk_main();
-	return 0;
+	return status;
 }
